import { system, world, EntityEquippableComponent, EquipmentSlot, ItemStack} from "@minecraft/server";
import {shields_loop} from './shield.js'

export function getScore(player, objective) {
	return world.scoreboard.getObjective(objective).getScore(player) ?? 0;
}
export function setScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).setScore(player, score) ?? 0;
}
export function addScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).addScore(player, score) ?? 0;
}
const w = world.getDimension("overworld");

world.beforeEvents.itemUseOn.subscribe(use =>{
	try{
		const p = use.source;
		const item = use.itemStack;
		const block = use.block;
		const l = [block.location.x, block.location.y, block.location.z]
		if(item.typeId == "new:wyvern_bone_hoe"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} farmland`)
			}
		}
		if(item.typeId == "new:skeleton_wyvern_hoe"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} farmland`)
			}
		}
		if(item.typeId == "new:wyvern_bone_shovel"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} grass_path`)
			}
		}
		let axe = ["acacia", "oak", "spruce", "jungle", "dark_oak", "birch", "mangrove", "cherry"]
		for(const ids of axe){
			if(item.typeId == "new:wyvern_bone_axe"){
				if(block.typeId.includes(`${ids}_log`)){
					p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} stripped_${ids}_log [] replace ${ids}_log`)
				}
			}
		}

	}catch(e){
	}
});
world.afterEvents.entitySpawn.subscribe(use =>{
	const e = use.entity;
	if(e.typeId.includes("_wyvern_egg")){
		const nameTag = e.typeId.replace("_wyvern_egg", "")
		e.runCommandAsync("scoreboard objectives add egg_time dummy")
		e.runCommandAsync("scoreboard players set @s egg_time 20")
		e.addTag(`${nameTag}`)
		
	}
});
const wyverns_type = ["fire", "ice", "ground", "poison", "wither", "ender"]
system.runInterval(() =>{
	try{
		for(const p of world.getPlayers()){
			wyvern_tamed_fly(p)
			bowUsing(p)
            detectEnchantements(p)
			shields_loop(p)
            p.runCommandAsync("function arrow")
			wyvern_wings(p)
			item_animations(p)
			for(const types of wyverns_type){
				wyvern_eggs(p, types)
				wyvern_inv(types)
			}
		}
	}catch(e){
	};
})

export function item_animations(p) {
	let equip = p.getComponent(EntityEquippableComponent.componentId);
	try{
		let item = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(item){
			if(item.hasTag("template:shield") && !p.hasTag("mainhand") && !p.hasTag("offhand") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_main_hand root 100`)
				p.addTag("mainhand")
			}
			if(!item.hasTag("template:shield") || p.hasTag("offhand") || !p.isSneaking){
				if(p.hasTag("mainhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
					p.removeTag("mainhand")
				}
			}
		}
		if(!item){
			if(p.hasTag("mainhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
				p.removeTag("mainhand")
			}
		}
	}catch(e){
		
	}
	try{
		let off = equip?.getEquipment(EquipmentSlot.Offhand);
		if(off){
			if(off.hasTag("template:shield")&&p.isSneaking){
				p.addTag("offhand")
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_off_hand root 100`)
			}
			if(!off.hasTag("template:shield")||!p.isSneaking){
				if(p.hasTag("offhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
					p.removeTag("offhand")
				}
			}
		}
		if(!off){
			if(p.hasTag("offhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
				p.removeTag("offhand")
			}
		}
	}catch(e){
		
	}
}


system.runInterval(()=>{			
	for(const types of wyverns_type){
		const wyverns_egg = w.getEntities({type: `new:${types}_wyvern_egg`})
		for(const we of wyverns_egg){
			const egg_time = getScore(we, "egg_time");
			if(egg_time > 0){
				we.runCommandAsync(`scoreboard players remove @s egg_time 1`)
			}
		}
	}
}, 1200)

function addEffect(p, effect, duration, level, particles){
	try{
		return p.addEffect(effect, duration*2, {amplifier: level, showParticles: particles}) ?? 0;
	}catch(e){

	}
}


world.afterEvents.itemUse.subscribe(use =>{
	try{
		const p = use.source;
        const bow = "_bow";
        const itemStack = use.itemStack;
        if(itemStack.typeId.toLowerCase().includes(bow)){
            p.addTag("using");
        } 
		if(itemStack.typeId.includes("_wyvern_wand") && !p.hasTag("cooldown")){
			let arrow = p.dimension.spawnEntity(itemStack.typeId, p.getHeadLocation());
			let projectile = arrow.getComponent('projectile');
			projectile.owner = p;
			projectile.shoot({x: p.getViewDirection().x*2, y: p.getViewDirection().y*2, z: p.getViewDirection().z*2});
			Durability(itemStack, p)
			p.addTag("cooldown")
			system.runTimeout(()=>{
				p.removeTag("cooldown")
			}, 60)
		}
	}catch(e){}
})

world.afterEvents.itemCompleteUse.subscribe(use =>{
		const p = use.source;
        const bow = "_bow";
        const item = use.itemStack;
        if(item.typeId.toLowerCase().includes(bow)){
            p.addTag("using");
        }
		if(item.typeId == "new:fire_essence"){
			addEffect(p, "fire_resistance", 1800, 4, false)
		}
		if(item.typeId == "new:potion_wyvern_slayer"){
			addEffect(p, "fire_resistance", 3600, 4, false)
			addEffect(p, "strenght", 3600, 4, false)
			addEffect(p, "resistance", 3600, 4, false)
			addEffect(p, "night_vision", 3600, 4, false)
			addEffect(p, "speed", 3600, 1, false)
		}
})

world.afterEvents.entityHurt.subscribe(hurt =>{
	try{
		const eHurt = hurt.hurtEntity;
		const damageSource = hurt.damageSource;
		const damage = hurt.damage;
		const cause = damageSource.cause;
		const damaging = damageSource.damagingEntity;
		if(damaging){
			if(damaging.typeId == "minecraft:player"){
				const equip = damaging.getComponent(EntityEquippableComponent.componentId);
				const hand = equip.getEquipment(EquipmentSlot.Mainhand);
				if(hand){
					if(hand.typeId == "new:poison_wyvern_dagger"){
						addEffect(eHurt, "poison", 40, 2, true)
					}
					if(hand.typeId == "new:ice_wyvern_dagger"){
						addEffect(eHurt, "slowness", 40, 2, true)
					}
					if(hand.typeId == "new:wither_wyvern_dagger"){
						addEffect(eHurt, "wither", 40, 2, true)
					}
				}
			}
		}
		if(eHurt){
			if(eHurt.typeId == "minecraft:player"&&eHurt.isSneaking){
				const equip = eHurt.getComponent(EntityEquippableComponent.componentId);
				const hand = equip.getEquipment(EquipmentSlot.Mainhand);
				const offhand = equip.getEquipment(EquipmentSlot.Offhand);
				if(hand){
					if(hand.typeId=="new:poison_wyvern_shield"){
						addEffect(damaging, "poison", 30, 1)
					}
					if(hand.typeId=="new:wither_wyvern_shield"){
						addEffect(damaging, "wither", 30, 1)
					}
					if(hand.typeId=="new:ice_wyvern_shield"){
						addEffect(damaging, "slowness", 30, 5)
					}
					if(hand.typeId=="new:fire_wyvern_shield"){
						damaging.setOnFire(5)
					}
					if(hand.typeId=="new:ground_wyvern_shield"){
						damaging.applyDamage(5)
					}
					if(hand.typeId=="new:ender_wyvern_shield"){
						damaging.runCommandAsync("tp ^^7^-3")
						eHurt.playSound("mob.shulker.teleport")
					}
				}
				if(offhand){
					if(offhand.typeId=="new:poison_wyvern_shield"){
						addEffect(damaging, "poison", 30, 1)
					}
					if(offhand.typeId=="new:wither_wyvern_shield"){
						addEffect(damaging, "wither", 30, 1)
					}
					if(offhand.typeId=="new:ice_wyvern_shield"){
						addEffect(damaging, "slowness", 30, 5)
					}
					if(offhand.typeId=="new:fire_wyvern_shield"){
						damaging.setOnFire(5)
					}
					if(offhand.typeId=="new:ground_wyvern_shield"){
						damaging.applyDamage(5)
					}
					if(offhand.typeId=="new:ender_wyvern_shield"){
						damaging.runCommandAsync("tp ^^7^-3")
						eHurt.playSound("mob.shulker.teleport")
					}
				}
			}
		}
	}catch(e){

	};
})
function wyvern_tamed_fly(p){
	if(p.hasTag("mount") && p.isJumping){
		p.runCommandAsync("effect @e[family=wyvern_tamed,r=4] levitation 1 5 true")
		p.runCommandAsync("event entity @e[family=wyvern_tamed,r=4] new:wyvern_can_fly")
	}
	if(!p.hasTag("mount")){
		p.runCommandAsync("event entity @e[family=wyvern_tamed,r=4] new:wyvern_not_can_fly")
	}
}
const tags= ["fire_wyvern", "bone_wyvern", "ice_wyvern", "wither_wyvern", "ground_wyvern", "poison_wyvern"]
function bowUsing(p){
    const equip = p.getComponent(EntityEquippableComponent.componentId);
    const hand = equip.getEquipment(EquipmentSlot.Mainhand);
    for(const bows of tags){
        if(p.hasTag("using")){
            if(hand.typeId == `new:${bows}_bow`){
                p.runCommandAsync(`playanimation @s[hasitem={item=new:${bows}_arrow}] animation.weapons.bow_and_arrow root 0.001 "!query.is_using_item"`)
            }
        }
    }
}
function detectEnchantements(p){
    const equip = p.getComponent(EntityEquippableComponent.componentId);
    const hand = equip.getEquipment(EquipmentSlot.Mainhand);
    if(hand){ 
        const enchantable = hand.getComponent("enchantable");
        for(const bows of tags){
            if(hand.typeId.includes(`${bows}_bow`)){
                if (enchantable) {
                    if (enchantable.getEnchantment("infinity")) {
                        p.addTag(`enchanted:infinity`)
                    }
                    if (!enchantable.getEnchantment("infinity")) {
                        p.removeTag(`enchanted:infinity`)
                    }
                    if(enchantable.getEnchantments().length === 0){
                        p.getTags().forEach(tags => {
                            if(tags.includes("enchanted:")){
                                p.removeTag(tags)
                            }
                        });
                    }
                }
            }
        }
        if(!hand.typeId.includes(`_bow`)){
            p.getTags().forEach(tag => {
                if(tag.includes("enchanted:")){
                    p.removeTag(tag)
                }
            });
        }
    }
    if (!hand) {
        p.getTags().forEach(tag => {
            if(tag.includes("enchanted:")){
                p.removeTag(tag)
            }
        });
    }
}

function wyvern_eggs(p, types){
	const wyverns_egg = w.getEntities({type: `new:${types}_wyvern_egg`})
	for(const we of wyverns_egg){
		const egg_time = getScore(we, "egg_time");
		if(!we.hasTag("shaked")){
			if(egg_time == 10){
				we.triggerEvent("new:egg_shaked")
				we.addTag("shaked")
			}
		}
		if(we.hasTag("shaked")){
			if(egg_time <= 0){
				we.triggerEvent("new:egg_transform")
			}
		}
		const view = p.getEntitiesFromViewDirection({maxDistance: 7});
		view.forEach((look) => {
			let eggTag;
			let firstWord;
			let eggName;
			if(look.entity.typeId == we.typeId){
				look.entity.getTags().forEach(t => {
					if (t.startsWith('new:')) {
						eggTag = `${t.replace("new:", "").split(",")}`
						firstWord = t[4].toUpperCase();
						eggName = `${firstWord}`+`${eggTag.replace(`${t[4]}`, "")}`
						p.onScreenDisplay.setActionBar(`§7--${eggName} Wyvern Egg--\n§6Hatching Time: §b${getScore(look.entity, "egg_time")} min`)
					}
				}); 
			}
		});
	}
}
function wyvern_inv(types){
	const wyverns = w.getEntities({type: `new:${types}_wyvern`})
	for(const wyvern of wyverns){
		const wInv = wyvern.getComponent("inventory").container;
		for(let i = 0;i<wInv.size;i++){
			const item = wInv.getItem(i)
			if(item){
				if(item.typeId=="new:wyvern_saddle" && i==0 && !wyvern.hasTag("saddled")){
					wyvern.triggerEvent("new:wyvern_saddled")
					wyvern.addTag("saddled")
				}
				if(item.typeId=="minecraft:chest" && i==1 && !wyvern.hasTag("chested")){
					wyvern.triggerEvent("wyvern_chested")
					wyvern.addTag("chested")
				}
				if(item.typeId=="new:wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
					wyvern.triggerEvent("wyvern_claws")
					wyvern.addTag("claws")
				}
				if(item.typeId=="new:gold_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
					wyvern.triggerEvent("gold_wyvern_claws")
					wyvern.addTag("claws")
				}
				if(item.typeId=="new:iron_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
					wyvern.triggerEvent("iron_wyvern_claws")
					wyvern.addTag("claws")
				}
				if(item.typeId=="new:diamond_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
					wyvern.triggerEvent("diamond_wyvern_claws")
					wyvern.addTag("claws")
				}
				if(item.typeId=="new:netherite_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
					wyvern.triggerEvent("netherite_wyvern_claws")
					wyvern.addTag("claws")
				}
			}
			if(!item && i==0 && wyvern.hasTag("saddled")){
				wyvern.triggerEvent("new:wyvern_unsaddled")
				wyvern.removeTag("saddled")
			}
			if(!item && i==1 && wyvern.hasTag("chested")){
				wyvern.triggerEvent("wyvern_unchested")
				wyvern.removeTag("chested")
			}
			if(!item && i==2 && wyvern.hasTag("claws")){
				wyvern.triggerEvent("wyvern_unclaws")
				wyvern.removeTag("claws")
			}
		}
	}
}

function wyvern_wings(p){
    const equip = p.getComponent(EntityEquippableComponent.componentId);
    const getWings = equip.getEquipment(EquipmentSlot.Offhand);
	const gamemode = p.getGameMode()
	const viewDir = p.getViewDirection()
	if(getWings){
		if(getWings.typeId.includes("_wyvern_wings")&&gamemode!="creative"&&gamemode!="spectator"){
			if(!p.hasTag("desactive_wings")){
				addEffect(p, "slow_falling", 5, 4, false)
			}
			if(p.isJumping&&!p.isSneaking&&!p.hasTag("desactive_wings")){
				const random = Math.floor(Math.random()*100)+1
				if(random==10){
					DurabilityOff(getWings, p)
				}
				p.runCommandAsync("playanimation @s animation.wyvern_wings.fly_root root 0.5")
				p.applyKnockback(viewDir.x * 2, viewDir.z * 2, 1, viewDir.y * 2);
			}
			if(!p.isJumping&&!p.hasTag("isSneaking")&&p.isSneaking&&p.hasTag("desactive_wings")){
				p.addTag("isSneaking")
				system.runTimeout(()=>{
					p.removeTag("desactive_wings")
					p.onScreenDisplay.setActionBar(`§a--Active--`)
				},60)
			}
			if(!p.isJumping&&p.isSneaking&&!p.hasTag("desactive_wings")&&!p.hasTag("isSneaking")){
				p.addTag("isSneaking")
				system.runTimeout(()=>{
					p.onScreenDisplay.setActionBar(`§c--Desactive--`)
					p.addTag("desactive_wings")
				},60)
			}
			if(!p.isSneaking){
				p.removeTag("isSneaking")
				p.onScreenDisplay.setActionBar(`§7-- Hold sneak to deactivate/active wings --`)
			}
		}
	}
}

world.afterEvents.playerSpawn.subscribe(spawn=>{
    world.gameRules.sendCommandFeedback=false;
	world.gameRules.commandBlockOutput= false;
})

world.afterEvents.itemStopUse.subscribe((stop) =>{
    try{
        const p = stop.source;
        const item = stop.itemStack;
        p.removeTag("using");
        for(const bows of tags){
            if(p.hasTag(`${bows}`)){
                if(item.typeId == `new:${bows}_bow` && p.hasTag("using")){
                    Durability(item, p)
                }
            }
        }
    }catch(e){

    }
})

world.afterEvents.projectileHitEntity.subscribe(hit=>{
    try{
        const p = hit.source;
        const projectile = hit.projectile;
        const target = hit.getEntityHit()
        if(projectile && target){
            for(const bows of tags){
                if(projectile.typeId.includes(`new:arrow_${bows}`)){
                    if(p.hasTag(`enchanted:infinity`)){
                        projectile.remove()
                    }
                }
            }
        }
    }catch(e){

    }
})

world.afterEvents.projectileHitBlock.subscribe(hit=>{
    try{
        const p = hit.source;
        const projectile = hit.projectile;
        const target = hit.getBlockHit()
        if(projectile && target){
            for(const bows of tags){
                if(projectile.typeId.includes(`new:arrow_${bows}`)){
                    if(p.hasTag(`enchanted:infinity`)){
                        projectile.remove()
                    }
                }
            }
        }
    }catch(e){

    }
})

export function Durability(tool, source) {
    try{
        system.run(()=>{
            const component = source.getComponent(EntityEquippableComponent.componentId);
            const durability = tool.getComponent("durability");
            const enchantable = tool.getComponent("enchantable");
            if(tool && durability !== undefined){
                let unbreakingLevel = 0;
                const damageAmount = 2;
                if (enchantable) {
                    for (const enchant of enchantable.getEnchantments()) {
                        if (enchant.type.id === "unbreaking") {
                            unbreakingLevel = enchant.level;
                            break;
                        }
                    }
                }if(durability.damage+2 < durability.maxDurability){
                    if(unbreakingLevel > 0) {
                        const chance = 1 / (unbreakingLevel + 1);
                        if (Math.random() < chance) {
                            durability.damage += damageAmount;
                        }
                    } if(unbreakingLevel == 0) {
                        durability.damage += damageAmount;
                    }
                    component.setEquipment(EquipmentSlot.Mainhand, tool)
                }else if(durability.damage+2 >= durability.maxDurability){
                    const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Mainhand, air)
                    source.playSound("random.break")
                }
            }
        })
    }catch(e){}
}

export function DurabilityOff(tool, source) {
    try{
        system.run(()=>{
            const component = source.getComponent(EntityEquippableComponent.componentId);
            const durability = tool.getComponent("durability");
            const enchantable = tool.getComponent("enchantable");
            if(tool && durability !== undefined){
                let unbreakingLevel = 0;
                const damageAmount = 2;
                if (enchantable) {
                    for (const enchant of enchantable.getEnchantments()) {
                        if (enchant.type.id === "unbreaking") {
                            unbreakingLevel = enchant.level;
                            break;
                        }
                    }
                }if(durability.damage+2 < durability.maxDurability){
                    if(unbreakingLevel > 0) {
                        const chance = 1 / (unbreakingLevel + 1);
                        if (Math.random() < chance) {
                            durability.damage += damageAmount;
                        }
                    } if(unbreakingLevel == 0) {
                        durability.damage += damageAmount;
                    }
                    component.setEquipment(EquipmentSlot.Offhand, tool)
                }else if(durability.damage+2 >= durability.maxDurability){
                    const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Offhand, air)
                    source.playSound("random.break")
					source.removeTag("desactive_wings")
					return;
                }
            }
        })
    }catch(e){}
}